using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Data;

namespace HexDecBinding
{
    public class CDataGridViewBinCell : System.Windows.Forms.DataGridViewTextBoxCell
    {
        protected override bool SetValue(int rowIndex, object value)
        {
            byte[] data = (byte[])value;
            List<byte> lst = null;
            if (value != null) lst = (ByteArray)data;
            string txt = "";
            if (lst != null) txt = lst.ToString();
            return base.SetValue(rowIndex, txt);
        }
        protected override object GetValue(int rowIndex)
        {
            object o = base.GetValue(rowIndex);
            //if (o.GetType() == (new byte[2]).GetType())
            if (o is byte[])
            {
                byte[] data = (byte[])o;
                ByteArray lst = (ByteArray)data;

                return (string)lst;
            }

            if (o is DBNull) return "null";

            string s = (string)o;

            if (s == null) return "";
            if (s.Length % 2 != 0) s = "0" + s;
            byte[] ba = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
            {
                ba[i] = Convert.ToByte(s.Substring(i, 2), 16);
            }
            if (ba.Length == 0) return "";
            return ba;
        }
    }

    class CDataGridViewBinColumn : System.Windows.Forms.DataGridViewTextBoxColumn
    {

        public CDataGridViewBinColumn()
        {
            this.CellTemplate = new CDataGridViewBinCell();
        }
    }

    class ByteArray : List<byte>
    {
        public static implicit operator ByteArray(byte[] data)
        {
            if (data == null) return null;
            ByteArray ba = new ByteArray();
            foreach (byte b in data)
                ba.Add(b);
            return ba;
        }
        public static explicit operator string(ByteArray ba)
        {
            string s = "";
            foreach (byte b in ba)
                s += Convert.ToString(b, 16).PadLeft(2, '0');
            return s;
        }
    }

    class Util
    {
        public static void SetupGVColumns(System.Windows.Forms.DataGridView dataGridView, System.Data.DataTable dt)
        {
            dataGridView.Columns.Clear();

            dataGridView.AutoGenerateColumns = false;
            byte[] ba = new byte[0];
            foreach (DataColumn col in dt.Columns)
            {
                System.Windows.Forms.DataGridViewColumn vc;
                if (col.DataType == ba.GetType())
                {
                    vc = new HexDecBinding.CDataGridViewBinColumn();
                }
                else
                {
                    vc = new DataGridViewColumn(new DataGridViewTextBoxCell());
                }
                vc.DataPropertyName = col.ColumnName;
                vc.HeaderText = col.ColumnName;
                dataGridView.Columns.Add(vc);
            }
        }
    }
}
