using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MySQLCSharpSample
{
    public partial class Form1 : Form
    {
        //OLEDB connection used throughout this class
        private System.Data.OleDb.OleDbConnection m_conn 
            = new System.Data.OleDb.OleDbConnection();
        public Form1()
        {
            InitializeComponent();
        }

        //On form load, establish an connection to MySQL database
        //When a connection is established, enable button to
        //allow user populate data
        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                //establish a db connection
                this.m_Read.Enabled = false;
                this.m_SP.Enabled = false;
                m_conn.ConnectionString = @"Provider=OleMySql.MySqlSource.1;"+
                    " Data Source=localhost,3306; Initial Catalog=test;" + 
                    " User ID=root; Password=MySQL";
                m_conn.Open();
                this.m_Create.Enabled = true;
                
                return;
            }
            catch (System.Data.OleDb.OleDbException ex)
            {
                MessageBox.Show(
                    "Failed to open connection to MySQL database. Error Details:"
                    + ex.Message);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            this.m_Create.Enabled = false;
            this.m_Read.Enabled = false;
        }

        /// <summary>
        /// Find if a table has been created by checking the table schema.
        /// </summary>
        /// <param name="sTable">sTable is the table name</param>
        /// <returns>return true if found, false otherwise</returns>
        private bool TableExist(string sTable)
        {
            System.Data.DataTable schema = 
                m_conn.GetOleDbSchemaTable(
                System.Data.OleDb.OleDbSchemaGuid.Tables, null);
            foreach (DataRow r in schema.Rows)
            {
                if (r.ItemArray.GetValue(2).ToString().ToLower() == 
                    sTable.ToLower() &&
                    r.ItemArray.GetValue(3).ToString().ToLower() == "table") 
                    return true;

            }
            return false;
        }

        //Create a test table if it does not exist. If it does exist,
        //delete data if any. Then insert data into the test table.
        private void m_Create_Click(object sender, EventArgs e)
        {
            try
            {
                //create command object
                System.Data.OleDb.OleDbCommand cmd
                    = new System.Data.OleDb.OleDbCommand();
                cmd.Connection = m_conn;

                //query string to create table 
                cmd.CommandText = @"CREATE TABLE FieldTest" +
                     "(ID INTEGER PRIMARY KEY, Name VARCHAR(20), " +
                     "LongDoc MEDIUMTEXT, Image BLOB, Date5 DATE, " +
                     "datetime6 DATETIME, time7 TIME, " +
                     "timestamp8 TIMESTAMP)";

                //if test has been created, delete all data
                if (this.TableExist("FieldTest"))
                    cmd.CommandText = "DELETE FROM FieldTest";
                cmd.ExecuteNonQuery();

                //insert data into FieldTest table
                cmd.CommandText = "INSERT INTO FieldTest " +
                    "(ID, Name, LongDoc, Image, Date5, datetime6, Time7, TimeStamp8)"
                    + " VALUES (?, ?, ?, ?, ?, ?, ?, ?);";
                System.Data.OleDb.OleDbParameter[] oParams
                    = new System.Data.OleDb.OleDbParameter[8];
                for (int i = 0; i < 8; i++)
                {
                    oParams[i] = new System.Data.OleDb.OleDbParameter();
                    cmd.Parameters.Add(oParams[i]);
                }

                
                //start transaction
                System.Data.OleDb.OleDbTransaction tran
                    = m_conn.BeginTransaction(); 
                cmd.Transaction = tran;
                byte[] ba = new byte[1000]; 
                for (int i = 0; i < 260; i++)
                {
                    oParams[0].Value = 100 + i;
                    oParams[0].DbType = DbType.Int32;
                    oParams[1].Value = "MyName " + i.ToString();
                    oParams[2].Value = "A long long long long long long text";
                    ba[i] = (byte)i;
                    oParams[3].Value = ba;
                    oParams[4].Value = System.DateTime.Now - new TimeSpan(i, 0, 0, 0);
                    oParams[4].DbType = DbType.Date;
                    oParams[5].Value = System.DateTime.Now
                       - new TimeSpan(i, i % 24, i % 60, i % 60);
                    oParams[5].DbType = DbType.DateTime;
                    oParams[6].Value = new TimeSpan(i % 24, i % 60, i % 60);
                    oParams[6].DbType = DbType.Time;
                    oParams[7].Value = System.DateTime.Now;
                    oParams[7].DbType = DbType.DateTime;
                    cmd.ExecuteNonQuery();
                }
                //at last commit transaction
                tran.Commit();

                //test all time related fields
                DateTimeTest();
                
                MessageBox.Show("Done");
                m_Read.Enabled = true;
                m_SP.Enabled = true;
            }
            catch (System.Data.OleDb.OleDbException ex)
            {
                MessageBox.Show(ex.Message);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
        }

        //Test date and time related data types
        private void DateTimeTest()
        {
            System.Data.OleDb.OleDbCommand cmd = 
                new System.Data.OleDb.OleDbCommand();
            cmd.Connection = m_conn;
            if (!this.TableExist("TimeTest"))
            {
                cmd.CommandText = 
                    "CREATE TABLE TimeTest (date0 DATE, time1 TIME," + 
                    " dateTime2 DATETIME, ts3 TIMESTAMP)";
            }
            else
            {
                cmd.CommandText = "delete from timeTest";
            }
            cmd.ExecuteNonQuery();

            System.Data.OleDb.OleDbParameter[] oParams = 
                new System.Data.OleDb.OleDbParameter[4];
            for (int i = 0; i < 4; i++)
            {
                oParams[i] = new System.Data.OleDb.OleDbParameter();
                oParams[i].Value = DateTime.Now;
                cmd.Parameters.Add(oParams[i]);
            }
            oParams[0].DbType = DbType.Date;
            oParams[1].Value = new TimeSpan(-80, 6, 2);
            oParams[1].DbType = DbType.Time;
            oParams[2].DbType = DbType.DateTime;
            oParams[3].DbType = DbType.DateTime;
            cmd.CommandText =
                "INSERT INTO TimeTest (date0, time1, dateTime2, ts3) values (?,?,?,?)";
            cmd.ExecuteNonQuery();

        }
        /// <summary>
        /// Create an stored procedure and test input and output parameters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void m_SP_Click(object sender, EventArgs e)
        {
            try
            {
                //Create a stored procedure
                System.Data.OleDb.OleDbCommand cmd =
                    new System.Data.OleDb.OleDbCommand();
                cmd.CommandText =
                    "DROP PROCEDURE IF EXISTS MyProc;CREATE PROCEDURE MyProc" +
                    " (param INT, OUT param1 INT, OUT Param2 DATE, OUT param3" + 
                    " DATETIME, OUT param4 TIME) BEGIN SET param1 = param*param;" + 
                    " SET param2='2000-10-10', param3='2001-12-31 01:02:03'," +
                    " param4='9:8:7'; END;";
                cmd.Connection = m_conn;
                cmd.ExecuteNonQuery();

                //call stored procedure with input and output parameters
                cmd.CommandText = "call MyProc(?, ?, ?, ?, ?)";

                System.Data.OleDb.OleDbParameter[] oParams = 
                    new System.Data.OleDb.OleDbParameter[5];
                for (int i = 0; i < 5; i++)
                {
                    oParams[i] = new System.Data.OleDb.OleDbParameter();
                    oParams[i].Direction = ParameterDirection.Output;
                    cmd.Parameters.Add(oParams[i]);
                }

                //setup input parameters
                oParams[0].Value = 10;
                oParams[0].DbType = DbType.Int32;
                oParams[0].Direction = ParameterDirection.Input;

                //setup output parameter types
                oParams[1].DbType = DbType.Int32;
                oParams[2].DbType = DbType.Date;
                oParams[3].DbType = DbType.DateTime;
                oParams[4].DbType  = DbType.Time;

                cmd.ExecuteNonQuery();
                
                //run debug to view output values
                System.Diagnostics.Debug.WriteLine(oParams[1].Value.ToString());
                System.Diagnostics.Debug.WriteLine(oParams[2].Value.ToString());
                System.Diagnostics.Debug.WriteLine(oParams[3].Value.ToString());
                System.Diagnostics.Debug.WriteLine(oParams[4].Value.ToString());

                MessageBox.Show("Done");
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
        }

        /// <summary>
        /// read data back from a FieldTest table and present it on a data grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void m_Read_Click(object sender, EventArgs e)
        {
            try
            {

                System.Data.OleDb.OleDbCommand cmd = new System.Data.OleDb.OleDbCommand();
                cmd.Connection = m_conn;
                cmd.CommandText = "select * from FieldTest";
                System.Data.OleDb.OleDbDataAdapter da =
                    new System.Data.OleDb.OleDbDataAdapter();
                da.SelectCommand = cmd;
                DataTable dt = new DataTable();
                da.Fill(dt);
                HexDecBinding.Util.SetupGVColumns(m_DGVResult, dt);
                this.m_DGVResult.DataSource = dt;
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
        }
    }
}